@@ -1,34 +1,13 @@
 from django.db import models
 from django.utils.translation import ugettext_lazy as _
 
-from model_utils.models import InheritanceCastModel, TimeStampedModel, StatusModel, TimeFramedModel
-from model_utils.managers import QueryManager, manager_from, InheritanceManager, PassThroughManager
+from model_utils.models import TimeStampedModel, StatusModel, TimeFramedModel
+from model_utils.managers import QueryManager, InheritanceManager, PassThroughManager
 from model_utils.fields import SplitField, MonitorField
 from model_utils import Choices
 
 
 
-class InheritParent(InheritanceCastModel):
-    non_related_field_using_descriptor = models.FileField(upload_to="test")
-    normal_field = models.TextField()
-    pass
-
-
-
-class InheritChild(InheritParent):
-    non_related_field_using_descriptor_2 = models.FileField(upload_to="test")
-    normal_field_2 = models.TextField()
-    pass
-
-
-
-class InheritChild2(InheritParent):
-    non_related_field_using_descriptor_3 = models.FileField(upload_to="test")
-    normal_field_3 = models.TextField()
-    pass
-
-
-
 class InheritanceManagerTestRelated(models.Model):
     pass
 
@@ -178,19 +157,6 @@ class FeaturedManager(models.Manager):
 
 
 
-class Entry(models.Model):
-    author = models.CharField(max_length=20)
-    published = models.BooleanField()
-    feature = models.BooleanField(default=False)
-
-    objects = manager_from(AuthorMixin, PublishedMixin, unpublished)
-    broken = manager_from(PublishedMixin, manager_cls=FeaturedManager)
-    featured = manager_from(PublishedMixin,
-                            manager_cls=FeaturedManager,
-                            queryset_cls=ByAuthorQuerySet)
-
-
-
 class DudeQuerySet(models.query.QuerySet):
     def abiding(self):
         return self.filter(abides=True)
@@ -225,3 +191,27 @@ class Dude(models.Model):
 
     objects = PassThroughManager(DudeQuerySet)
     abiders = AbidingManager()
+
+
+class Car(models.Model):
+    name = models.CharField(max_length=20)
+    owner = models.ForeignKey(Dude, related_name='cars_owned')
+
+    objects = PassThroughManager(DudeQuerySet)
+
+
+class SpotQuerySet(models.query.QuerySet):
+    def closed(self):
+        return self.filter(closed=True)
+
+    def secured(self):
+        return self.filter(secure=True)
+
+
+class Spot(models.Model):
+    name = models.CharField(max_length=20)
+    secure = models.BooleanField(default=True)
+    closed = models.BooleanField(default=False)
+    owner = models.ForeignKey(Dude, related_name='spots_owned')
+
+    objects = PassThroughManager.for_queryset_class(SpotQuerySet)()
