@@ -1,27 +1,24 @@
 from __future__ import with_statement
 
-import pickle, sys, warnings
+import pickle
 
 from datetime import datetime, timedelta
 
-import django
-from django.test import TestCase
 from django.db import models
 from django.db.models.fields import FieldDoesNotExist
 from django.core.exceptions import ImproperlyConfigured
-
-from django.contrib.contenttypes.models import ContentType
+from django.test import TestCase
 
 from model_utils import Choices
 from model_utils.fields import get_excerpt, MonitorField
-from model_utils.managers import QueryManager, manager_from
+from model_utils.managers import QueryManager
 from model_utils.models import StatusModel, TimeFramedModel
 from model_utils.tests.models import (
-    InheritParent, InheritChild, InheritChild2, InheritanceManagerTestRelated,
+    InheritanceManagerTestRelated,
     InheritanceManagerTestParent, InheritanceManagerTestChild1,
     InheritanceManagerTestChild2, TimeStamp, Post, Article, Status,
     StatusPlainTuple, TimeFrame, Monitored, StatusManagerAdded,
-    TimeFrameManagerAdded, Entry, Dude, SplitFieldAbstractParent)
+    TimeFrameManagerAdded, Dude, SplitFieldAbstractParent, Car, Spot)
 
 
 
@@ -184,6 +181,10 @@ class ChoicesTests(TestCase):
         self.assertEquals(tuple(self.STATUS), (('DRAFT', 'DRAFT'), ('PUBLISHED', 'PUBLISHED')))
 
 
+    def test_len(self):
+        self.assertEqual(len(self.STATUS), 2)
+
+
     def test_repr(self):
         self.assertEquals(repr(self.STATUS),
                           "Choices("
@@ -225,6 +226,10 @@ class LabelChoicesTests(ChoicesTests):
         self.assertEquals(self.STATUS.DRAFT, 'DRAFT')
 
 
+    def test_len(self):
+        self.assertEqual(len(self.STATUS), 3)
+
+
     def test_repr(self):
         self.assertEquals(repr(self.STATUS),
                           "Choices("
@@ -257,6 +262,10 @@ class IdentifierChoicesTests(ChoicesTests):
         self.assertEquals(self.STATUS.DRAFT, 0)
 
 
+    def test_len(self):
+        self.assertEqual(len(self.STATUS), 3)
+
+
     def test_repr(self):
         self.assertEquals(repr(self.STATUS),
                           "Choices("
@@ -265,106 +274,86 @@ class IdentifierChoicesTests(ChoicesTests):
                           "(2, 'DELETED', 'is deleted'))")
 
 
-class InheritanceCastModelTests(TestCase):
+
+class InheritanceManagerTests(TestCase):
     def setUp(self):
-        self.parent = InheritParent.objects.create()
-        self.child = InheritChild.objects.create()
+        self.child1 = InheritanceManagerTestChild1.objects.create()
+        self.child2 = InheritanceManagerTestChild2.objects.create()
 
 
-    def test_parent_real_type(self):
-        self.assertEquals(self.parent.real_type,
-                          ContentType.objects.get_for_model(InheritParent))
+    def get_manager(self):
+        return InheritanceManagerTestParent.objects
 
 
-    def test_child_real_type(self):
-        self.assertEquals(self.child.real_type,
-                          ContentType.objects.get_for_model(InheritChild))
+    def test_normal(self):
+        self.assertEquals(set(self.get_manager().all()),
+                          set([
+                    InheritanceManagerTestParent(pk=self.child1.pk),
+                    InheritanceManagerTestParent(pk=self.child2.pk),
+                    ]))
 
 
-    def test_cast(self):
-        obj = InheritParent.objects.get(pk=self.child.pk).cast()
-        self.assertEquals(obj.__class__, InheritChild)
+    def test_select_all_subclasses(self):
+        self.assertEquals(
+            set(self.get_manager().select_subclasses()),
+            set([self.child1, self.child2]))
 
 
-    # @@@ Use proper test skipping once Django 1.2 is minimum supported version.
-    if sys.version_info >= (2, 6):
-        # @@@ catch_warnings only available in Python 2.6 and newer
-        def test_pending_deprecation(self):
-            with warnings.catch_warnings(record=True) as w:
-                warnings.simplefilter("always")
-                InheritParent()
-                self.assertEqual(len(w), 1)
-                assert issubclass(w[-1].category, PendingDeprecationWarning)
+    def test_select_specific_subclasses(self):
+        self.assertEquals(
+            set(self.get_manager().select_subclasses(
+                    "inheritancemanagertestchild1")),
+            set([self.child1,
+                 InheritanceManagerTestParent(pk=self.child2.pk)]))
+
+    def test_get_subclass(self):
+        self.assertEquals(
+            self.get_manager().get_subclass(pk=self.child1.pk),
+            self.child1)
 
 
-
-class InheritanceCastQuerysetTests(TestCase):
+class InheritanceManagerRelatedTests(InheritanceManagerTests):
     def setUp(self):
-        self.child = InheritChild.objects.create()
-        self.child2 = InheritChild2.objects.create()
+        self.related = InheritanceManagerTestRelated.objects.create()
+        self.child1 = InheritanceManagerTestChild1.objects.create(
+            related=self.related)
+        self.child2 = InheritanceManagerTestChild2.objects.create(
+            related=self.related)
 
 
-    def test_cast_manager(self):
-        self.assertEquals(set(InheritParent.objects.cast()),
-                          set([self.child, self.child2]))
+    def get_manager(self):
+        return self.related.imtests
 
 
-    def test_cast(self):
-        parent = InheritParent.objects.create()
-        obj = InheritParent.objects.filter(pk=self.child.pk).cast()[0]
-        self.assertEquals(obj.__class__, InheritChild)
-        self.assertEquals(set(InheritChild2.objects.all().cast()),
-                          set([self.child2]))
-        self.assertEquals(set(InheritParent.objects.all().cast()),
-                          set([parent, self.child, self.child2]))
+    def test_get_method_with_select_subclasses(self):
+        self.assertEqual(
+            InheritanceManagerTestParent.objects.select_subclasses().get(
+                id=self.child1.id),
+            self.child1)
 
 
-
-# @@@ Use proper test skipping once 1.2 is minimum supported version.
-if django.VERSION >= (1, 2):
-    class InheritanceManagerTests(TestCase):
-        def setUp(self):
-            self.child1 = InheritanceManagerTestChild1.objects.create()
-            self.child2 = InheritanceManagerTestChild2.objects.create()
+    def test_annotate_with_select_subclasses(self):
+        qs = InheritanceManagerTestParent.objects.select_subclasses().annotate(
+            models.Count('id'))
+        self.assertEqual(qs.get(id=self.child1.id).id__count, 1)
 
 
-        def get_manager(self):
-            return InheritanceManagerTestParent.objects
+    def test_annotate_with_named_arguments_with_select_subclasses(self):
+        qs = InheritanceManagerTestParent.objects.select_subclasses().annotate(
+            test_count=models.Count('id'))
+        self.assertEqual(qs.get(id=self.child1.id).test_count, 1)
 
 
-        def test_normal(self):
-            self.assertEquals(set(self.get_manager().all()),
-                              set([
-                        InheritanceManagerTestParent(pk=self.child1.pk),
-                        InheritanceManagerTestParent(pk=self.child2.pk),
-                        ]))
+    def test_annotate_before_select_subclasses(self):
+        qs = InheritanceManagerTestParent.objects.annotate(
+            models.Count('id')).select_subclasses()
+        self.assertEqual(qs.get(id=self.child1.id).id__count, 1)
 
 
-        def test_select_all_subclasses(self):
-            self.assertEquals(
-                set(self.get_manager().select_subclasses()),
-                set([self.child1, self.child2]))
-
-
-        def test_select_specific_subclasses(self):
-            self.assertEquals(
-                set(self.get_manager().select_subclasses(
-                        "inheritancemanagertestchild1")),
-                set([self.child1,
-                     InheritanceManagerTestParent(pk=self.child2.pk)]))
-
-
-    class InheritanceManagerRelatedTests(InheritanceManagerTests):
-        def setUp(self):
-            self.related = InheritanceManagerTestRelated.objects.create()
-            self.child1 = InheritanceManagerTestChild1.objects.create(
-                related=self.related)
-            self.child2 = InheritanceManagerTestChild2.objects.create(
-                related=self.related)
-
-
-        def get_manager(self):
-            return self.related.imtests
+    def test_annotate_with_named_arguments_before_select_subclasses(self):
+        qs = InheritanceManagerTestParent.objects.annotate(
+            test_count=models.Count('id')).select_subclasses()
+        self.assertEqual(qs.get(id=self.child1.id).test_count, 1)
 
 
 
@@ -514,13 +503,13 @@ class QueryManagerTests(TestCase):
 
 
 
-# @@@ Use proper test skipping once Django 1.2 is minimum supported version.
 try:
     from south.modelsinspector import introspector
 except ImportError:
     introspector = None
 
-if introspector is not None:
+# @@@ use skipUnless once Django 1.3 is minimum supported version
+if introspector:
     class SouthFreezingTests(TestCase):
         def test_introspector_adds_no_excerpt_field(self):
             mf = Article._meta.get_field('body')
@@ -536,54 +525,6 @@ if introspector is not None:
 
 
 
-class ManagerFromTests(TestCase):
-    def setUp(self):
-        Entry.objects.create(author='George', published=True)
-        Entry.objects.create(author='George', published=False)
-        Entry.objects.create(author='Paul', published=True, feature=True)
-
-
-    def test_chaining(self):
-        self.assertEqual(Entry.objects.by_author('George').published().count(),
-                         1)
-
-
-    def test_function(self):
-        self.assertEqual(Entry.objects.unpublished().count(), 1)
-
-
-    def test_typecheck(self):
-        self.assertRaises(TypeError, manager_from, 'somestring')
-
-
-    def test_custom_get_query_set(self):
-        self.assertEqual(Entry.featured.published().count(), 1)
-
-
-    def test_cant_reconcile_qs_class(self):
-        self.assertRaises(TypeError, Entry.broken.all)
-
-
-    def test_queryset_pickling_fails(self):
-        qs = Entry.objects.all()
-        def dump_load():
-            pqs = pickle.dumps(qs)
-            pickle.loads(pqs)
-        self.assertRaises(pickle.PicklingError, dump_load)
-
-
-    # @@@ Use proper test skipping once Django 1.2 is minimum supported version.
-    if sys.version_info >= (2, 6):
-        # @@@ catch_warnings only available in Python 2.6 and newer
-        def test_pending_deprecation(self):
-            with warnings.catch_warnings(record=True) as w:
-                warnings.simplefilter("always")
-                manager_from()
-                self.assertEqual(len(w), 1)
-                assert issubclass(w[-1].category, PendingDeprecationWarning)
-
-
-
 class PassThroughManagerTests(TestCase):
     def setUp(self):
         Dude.objects.create(name='The Dude', abides=True, has_rug=False)
@@ -612,3 +553,25 @@ class PassThroughManagerTests(TestCase):
         saltyqs = pickle.dumps(qs)
         unqs = pickle.loads(saltyqs)
         self.assertEqual(unqs.by_name('The Dude').count(), 1)
+
+
+    def test_queryset_not_available_on_related_manager(self):
+        dude = Dude.objects.by_name('Duder').get()
+        Car.objects.create(name='Ford', owner=dude)
+        self.assertFalse(hasattr(dude.cars_owned, 'by_name'))
+
+
+class CreatePassThroughManagerTests(TestCase):
+    def setUp(self):
+        self.dude = Dude.objects.create(name='El Duderino')
+        Spot.objects.create(name='The Crib', owner=self.dude, closed=True,
+                            secure=True)
+
+    def test_reverse_manager(self):
+        self.assertEqual(self.dude.spots_owned.closed().count(), 1)
+
+    def test_related_queryset_pickling(self):
+        qs = self.dude.spots_owned.closed()
+        pickled_qs = pickle.dumps(qs)
+        unpickled_qs = pickle.loads(pickled_qs)
+        self.assertEqual(unpickled_qs.secured().count(), 1)
